###########################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
from ZSI import ParsedSoap, SoapWriter
from ZSI.address import Address
from ZSI.schema import GED

namespaceURI = Address().wsAddressURI

def save(fd, pyobj, typecode=None):
    """Save pyobj to file as XML
    fd -- file descriptor
    pyobj --
    typecode -- optional typecode
    """
    sw = SoapWriter(envelope=False)
    sw.serialize(pyobj, typecode=getattr(pyobj, 'typecode', typecode))
    fd.write(str(sw))


def load(fd, typecode):
    """Load pyobj from fd
    fd -- file descriptor
    typecode -- 
    """
    ps = ParsedSoap(fd.read(), envelope=False)
    return ps.Parse(typecode)


class EndpointReference(object):
    def __init__(self):
        self._Address = None
        self._ReferenceProperties = None

    @staticmethod
    def save(fd, pyobj, typecode=GED(namespaceURI, 'EndpointReference')):
        save(fd, pyobj, typecode)

    @staticmethod
    def load(fd, typecode=GED(namespaceURI, 'EndpointReference')):
        return load(fd, typecode)
    
    def getAddress(self, value):
        return self._Address
    def setAddress(self, value):
        self._Address = value
    Address = property(getAddress, setAddress, None, 
        'local Address')

    def getReferenceProperties(self, value):
        return self._ReferenceProperties
    def setReferenceProperties(self, value):
        self._ReferenceProperties = value
    ReferenceProperties = property(getReferenceProperties, setReferenceProperties, None, 
        'local ReferenceProperties')

    def new_ReferenceProperties(self):
        return ReferenceProperties()

## retain old mapping
EndPointReference = EndpointReference


class ReferenceProperties(object):
    def setAny(self, value):
        self._any = value
    def getAny(self, value):
        return self._any
    Any = property(getAny, setAny, None,
        'wildcard')


class AddressingUtils:

    @staticmethod
    def createEndpointReference(ctx, pyobj=None):
        epr = pyobj or EndpointReference()
        epr.Address = ctx.getAddress()
        resourceID = ctx.getResourceID()
        if resourceID:
            epr.ReferenceProperties = refprop = epr.new_ReferenceProperties()
            refprop.Any = [resourceID]
        return epr

    @staticmethod
    def getResourceID(ps, typecode):
        '''
        typecode -- representing the ResourceID expecting in WS-Address
           Header.
        '''
        d = ps.ParseHeaderElements((typecode,))
        return d[(typecode.nspname,typecode.pname)]

