############################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
import warnings
from zope.interface import Interface
from twisted.plugin import getPlugins as _getPlugins, IPlugin

def SetUpDefaults(**kw):
    """keyword argument names are plugin names,
    the values are the interface.
    """
    for k,v in kw.items(): 
       #__import__(item[0], globals(), locals(), item[1]) 
       SetDefaultPlugin(k, v)

    return


DEFAULT = {}
def SetDefaultPlugin(interface, default):
    """Set a default plugin to be returned via a
    interface name.  Store everything as lower.

    @param interface: Name of interface class.

    @param default: Name of default plugin.

    @return: None
    """
    if not isinstance(interface, basestring):
       iname = interface.__name__.lower()

       if not (isinstance(default, basestring) or
           (interface.providedBy(default) and IPlugin.providedBy(default))):
           raise Invalid, 'Plugin does not provide specified interface'
       
    else:
        iname = interface.lower()

    if isinstance(default, basestring):
        DEFAULT[iname] = default.lower()
        return

    DEFAULT[iname] = default


def GetPlugins(interface):
    import pyGridWare.plugins
    return _getPlugins(interface, pyGridWare.plugins)


def GetPlugin(interface):
    """Retrieve a plugin implementing the given interface beneath the 
    pyGridWare.plugins module.

    @param interface: An interface class.  Only plugins which
    implement this interface will be returned.

    @return: An iterator of plugins.
    """
    import pyGridWare.plugins
    default = DEFAULT.get(interface.__name__.lower())
    plug = None

    it = GetPlugins(interface)
    for plug in it:
        if default is None: 
            try: 
                it.next()
            except StopIteration: 
                pass
            else: 
                warnings.warn(
                   'Found multiple %s plugins and no default was specified, choosing plugin %s'\
                   %(interface, plug),
                )
            break
        if default is plug or default == plug.__name__.lower():
            break
    else:
        if plug is None:
            raise RuntimeError,\
                'failed to find any plugins for interface %s' %(interface)

        raise RuntimeError,\
            'failed to find a plugin named %s for interface %s' %(default,interface)

    return plug




