############################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
import os, sys
from zope.interface import implements
from twisted.python import usage
from twisted.plugin import IPlugin
from twisted.application.service import IServiceMaker
from twisted.application import internet
from twisted.protocols.policies import WrappingFactory
from twisted.web.server import Site
from twisted.web.resource import Resource
from twisted.web.script import ResourceScript
from twisted.python import log

# Need Chunked HTTP for globus-4 interop
from pyGridWare.utility.http import ChunkedHTTPChannel
from ServiceMaker import setUpDropIns, setUpNotification


class GSIOptions(usage.Options):
    optParameters = [
        ["port", "p", 8080, "The port number to listen on."],
        ["interface", "i", "localhost", "The interface to listen on."],
        ["gsipath", "g", os.path.join('/etc', 'grid-security',), "Path to GSI Directory Structure"],
        ["tree", "t", '.'.join((setUpDropIns.__module__,
                                setUpDropIns.__name__)), 
         "initialize function, called with the root twisted.web.Resource"],
    ]

    def opt_help(self):
        """Display this help and exit."""
        print self.__str__()
        sys.exit(0)


class GSIServiceMaker(object):
    implements(IServiceMaker, IPlugin)
    tapname = "wsrf-gsi"
    description = "WSRF Service with GSI Security"
    options = GSIOptions

    def makeService(self, options):
        """
        Construct a GSIServer 
        """
        from pyGridWare.utility.gsi import get_calist, GSIServer, GSIServerContextFactory
        from pyGridWare.utility.m2crypto import GridMapAuthz
        from pyGridWare.utility.log import gridLog

        gridLog(event='pyGridWare.plugins.GSIServiceMaker.makeService')

        root = Resource()
        Site.protocol = ChunkedHTTPChannel
        siteFactory = Site(root)

        gsipath = options['gsipath']
        capath = os.path.join(gsipath, 'certificates')
        key = os.path.join(gsipath, 'containerkey.pem')
        cert = os.path.join(gsipath, 'containercert.pem')

        authz = GridMapAuthz(gridmap=os.path.join(gsipath, 'grid-mapfile'))
        contextFactory = GSIServerContextFactory(\
                     privateKeyFileName=key,
                     certificateFileName=cert,
                     calist=get_calist(capath),
                     capath=capath)

        service = GSIServer(int(options["port"]), siteFactory, contextFactory=contextFactory, 
                      interface=options["interface"], postConnectionCheck=authz)

        root.__class__.server = service

        # Initialize Root Function
        tree = options['tree']
        log.msg('call initialization function: %s' %tree, debug=True)
        
        args = tree.split('.')
        mod = __import__('.'.join(args[:-1]))
        for attr in args[1:]:
            mod = getattr(mod, attr)
            
        func = mod    
        assert callable(func), 'initialization function must be callable'
        func(root)

        return service


# Now construct an object which *provides* the relevant interfaces
# The name of this variable is irrelevant, as long as there is *some*
# name bound to a provider of IPlugin and IServiceMaker.

gsiServiceMaker = GSIServiceMaker()



