###########################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
import os, time
import weakref
from ConfigParser import SafeConfigParser as ConfigParser,\
    NoSectionError, NoOptionError

from zope.interface import classProvides, implements
from twisted.plugin import IPlugin
from twisted.python import log
from pyGridWare.wsrf.faults.PropertiesFaults import ResourceUnknownFault
from pyGridWare.resource.interfaces import IResourceHome, IPersistent, IResourceHomeFactory


class PersistentResourceHome:
    '''Persistent Resource Home, uses a file w/ConfigParser syntax 
    to save ResourceProperties of all resource context properties. 
    All instances will share a single file.
    '''
    implements(IResourceHome)
    classProvides(IPlugin, IPersistent, IResourceHomeFactory)
    
    resourceID = 7777
    stateDir = 'persistent_resources'
    try: os.makedirs(stateDir)
    except: pass
    loaded = False
    
    def __init__(self):
        self.__avatarId = None
        self.resources = {}

    @classmethod
    def _getfilename(cls, manager):
            return os.path.join(cls.stateDir, (manager.__class__.__module__ + manager.__class__.__name__).replace('.','_') + '.txt')

    def getResourceManager(self):
        """ResourceHomeManager instance managing this resource
        """
        return self.manager()
        
    def setResourceManager(self, manager):
        """ResourceHomeManager instance managing this resource
        """
        self.manager = weakref.ref(manager)
        
    # IResourceHome
    def add(self, ctx, key=None):
        '''set a Nofication Listener in the topicListeners for each
        resource created.  Whenever a RP is changed --> callback.
        '''
        if key: ctx.resourceID = key
        key = ctx.resourceID
        if int(key) > PersistentResourceHome.resourceID:
            PersistentResourceHome.resourceID = int(key)
            self.create()
        self.resources[str(key)] = ctx
        
        # Notify me of changes
        def cb(*args): PersistentResourceHome.save(self, key)
        ctx.properties.topicListeners.append(cb)


    def create(self):
        key = PersistentResourceHome.resourceID 
        PersistentResourceHome.resourceID += 1
        return key

    def delete(self, key):
        del self.resources[str(key)]
        PersistentResourceHome.save(self, key)

    def get(self, key):
        if self.resources.has_key(str(key)) is False:
            log.msg('%s key(%s) not in resources: %s' %(self, key,self.resources),
                    debug=True)
            raise ResourceUnknownFault(\
                    timestamp=time.gmtime(),
                    originator=None,
                    errorcode=None,
                    description=None,
                    faultcause=None
            )
        return self.resources[str(key)]
    
    def getResources(self):
        return self.resources.values()
        
    def setTerminationTime(self, key, terminationTime):
        self.resources[str(key)].TerminationTime = terminationTime
        
    def setAvatarId(self, avatarId):
        if self.__avatarId is not None:
            raise RuntimeError, 'avatarId can only be set if None.'
        
        self.__avatarId = avatarId
    def getAvatarId(self):
        return self.__avatarId
    
    # IPersistent
    @classmethod
    def load(cls, manager):
        print ">LOAD "*20
        if cls.loaded:
            raise RuntimeError, 'resource already loaded.'
        
        homes = []
        cp = ConfigParser()
        try:
            cp.readfp(open(cls._getfilename(manager), 'r'))
        except IOError:
            return homes
        
        append = homes.append
        sections = cp.sections()
        if len(sections) is 0: 
            sections = (cls.ANONYMOUS,)
        
        for section in sections:
            home = cls()
            append(home)
            home.setAvatarId(section)
            home.setResourceManager(manager)
            options = cp.options(section)
            
            for resourceId in options:
                d = cp.get(section, resourceId)
                d = eval(d)
                
                ctx_mod,ctx_class =  d.get('context')
                prop_mod,prop_class,value =  d.get('properties')
                url = d.get('url')

                # import context & properties
                for i in ('from %s import %s' %(ctx_mod,ctx_class), \
                    'from %s import %s' %(prop_mod,prop_class)):
                    exec(i)
                    
                # create instance of context & properties
                ctx_class = eval(ctx_class)
                prop_class = eval(prop_class)
                ctx = ctx_class(prop_class)
                ctx.properties.setUp(value)
                ctx.resourceID = int(resourceId)
                ctx.url = url
                
                home.add(ctx)
                if int(ctx.resourceID) == cls.resourceID:
                    home.create()
                    
        cls.loaded = True
        return homes
    
    ANONYMOUS = "Anonymous"
    @classmethod
    def save(cls, home, key=None):
        '''save ResourceProperties corresponding to key, or all 
        if key is None.  
        manager -- <ResourceHomeManager>
        home -- <ResourceHome>
        key -- resource ID
        '''
        print "SAVE "*20
        filename = cls._getfilename(manager=home.getResourceManager())
        section = avatarId = home.getAvatarId() or cls.ANONYMOUS
        cp = ConfigParser()
        try:
            fp = open(filename)
        except IOError:
            cp.add_section(section)
        else:
            cp.readfp(fp)
            fp.close()
            if cp.has_section(section) is False:
                cp.add_section(section)
            
        for resourceId,context in home.resources.items():
            if context is not None:
                cp.set(section, resourceId, 
                   "{'url':'%s', 'context':('%s','%s'), 'properties':('%s','%s','%s')}" %(
                   context.url, 
                   context.__class__.__module__, 
                   context.__class__.__name__, 
                   context.properties.__class__.__module__,
                   context.properties.__class__.__name__,
                   str(context.properties))
                   )
                
        cp.write(open(filename, 'w'))
        

