############################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
import os, sys
from zope.interface import implements
from twisted.python import usage
from twisted.plugin import IPlugin
from twisted.application.service import IServiceMaker
from twisted.application import internet
from twisted.protocols.policies import WrappingFactory
from twisted.web.server import Site
from twisted.web.resource import Resource
from twisted.web.script import ResourceScript
from twisted.python import log

# Need Chunked HTTP for globus-4 interop
from pyGridWare.utility.http import ChunkedHTTPChannel
from ServiceMaker import setUpDropIns, setUpNotification


_abs = lambda f: os.path.abspath(f)
class SSLOptions(usage.Options):
    optParameters = [
        ["port", "p", 8080, "The port number to listen on."],
        ["interface", "i", "localhost", "The interface to listen on."],
        ["cert", "c", None, "server certficate"],
        ["key", "k", None, "server private key"],
        ["calist", "l", '', "list CA certficates"],
        ["tree", "t", '.'.join((setUpDropIns.__module__,
                                setUpDropIns.__name__)), 
         "initialize function, called with the root twisted.web.Resource"],

    ]
    
    def opt_help(self):
        """Display this help and exit."""
        print self.__str__()
        print 'NOTES:'
        print '    RPY scripts in the "services" directory will be discovered ' +\
        'upon receipt of a request directed at PATH "/wsrf/services/*"' 
        print
        sys.exit(0)



class SSLServiceMaker(object):
    implements(IServiceMaker, IPlugin)
    tapname = "wsrf-ssl"
    description = "SSL security using M2Crypto"
    options = SSLOptions

    def makeService(self, options):
        """
        Construct a TCPServer
        """
        from M2Crypto import SSL
        from pyGridWare.utility.m2crypto import\
            TLSProtocolSessionCachingWrapper as TLSProtocolWrapper
        from pyGridWare.utility.log import gridLog

        gridLog(event='pyGridWare.plugins.SSLServiceMaker.makeService')

        root = Resource()
        Site.protocol = ChunkedHTTPChannel
        siteFactory = Site(root)

        # TODO: How should this get configured???
        cert = options["cert"]
        key = options["key"]
        calist = options["calist"].split(",")
        #capath = abs(os.path.join(os.curdir, 'etc', 'certificates'))

        # SSL CONTEXT 
        ctx = SSL.Context('sslv3')
        class ContextFactory:
            def getContext(self): return ctx
    
        ctx.set_cipher_list("NULL-MD5:ALL:!ADH:!EXP:@STRENGTH")
        try:
            ctx.load_cert(cert, key)
        except SSL.SSLError, ex:
            ex.args = ('Require valid server cert/key files: (%s, %s)' %(cert,key),)
            raise

        ctx.set_allow_unknown_ca(False)
        ctx.set_verify(SSL.verify_peer, 10)

        for ca in filter(lambda i:i, calist): 
            #ctx.load_verify_locations(cafile=ca, capath=capath)
            ctx.load_verify_locations(cafile=ca)

        wrapFactory = WrappingFactory(siteFactory)
        wrapFactory.protocol.TLS = True
        wrapFactory.protocol = lambda factory, wrappedProtocol: \
            TLSProtocolWrapper(factory,
                               wrappedProtocol,
                               startPassThrough=0,
                               client=0,
                               contextFactory=ContextFactory(),
                               postConnectionCheck=None)

        service = internet.TCPServer(int(options["port"]), wrapFactory, 
                     interface=options["interface"])

        root.__class__.server = service

        # Initialize Root Function
        tree = options['tree']
        log.msg('call initialization function: %s' %tree, debug=True)
        
        args = tree.split('.')
        mod = __import__('.'.join(args[:-1]))
        for attr in args[1:]:
            mod = getattr(mod, attr)
            
        func = mod    
        assert callable(func), 'initialization function must be callable'
        func(root)
        
        return service
 

