############################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
import os, sys
from zope.interface import implements
from twisted.python import usage
from twisted.plugin import IPlugin
from twisted.application.service import IServiceMaker
from twisted.application import internet
from twisted.protocols.policies import WrappingFactory
from twisted.web.server import Site
from twisted.web.resource import Resource
from twisted.python import log
from pyGridWare import plugin

# Need Chunked HTTP for globus-4 interop
from pyGridWare.utility.http import ChunkedHTTPChannel


def setUpNotification(root):
    """adds a subscription manager service, required for WS-Notificaion support.
    
    /wsrf/core/SubscriptionManager
    """
    from ZSI.address import Address
    from pyGridWare.wsrf.notification.SubscriptionManager import SubscriptionManager
    from pyGridWare.services.SubscriptionManager import SubscriptionManager as SMServiceBinding
 
    core = Resource()
    root.putChild('wsrf', Resource())
    root.getStaticEntity('wsrf').putChild('core', core)
    wsrf = Resource()

    # Discover WS-Address URI
    addr = Address()
    addr.setUp()

    # Initialize singleton
    manager = SubscriptionManager.getInstance(SMServiceBinding,
                      {'wsAddressURI':addr.wsAddressURI,
                       'contextFactory':core.server.args[1]},)
    core.putChild('SubscriptionManagerService', manager)  
    return manager 
    
def setUpDropIns(root):
    from twisted.web.static import File
    from twisted.web.script import ResourceScript
    
    setUpNotification(root)
    
    # Drop in directory "services"
    dropins = File("services")
    dropins.ignoreExt(".rpy")
    dropins.processors = {".rpy": ResourceScript}
    
    root.getStaticEntity('wsrf').putChild('services', dropins)
    return dropins


class SimpleOptions(usage.Options):
    optParameters = [
        ["port", "p", 8080, "The port number to listen on."],
        ["interface", "i", "localhost", "The interface to listen on."],
        ["tree", "t", '.'.join((setUpDropIns.__module__,
                                setUpDropIns.__name__)), 
         "initialize function, called with the root twisted.web.Resource"],
    ]

    def opt_help(self):
        """Display this help and exit."""
        print self.__str__()
        print 'NOTES:'
        print '    RPY scripts in the "services" directory will be discovered ' +\
        'upon receipt of a request directed at PATH "/wsrf/services/*"' 
        print
        sys.exit(0)

    def opt_version(self):
        from pyGridWare import version
        from twisted import copyright
        print "pyGridWare version: %d.%d.%d" % version.Version
        print "\tTwisted version:", copyright.version
        sys.exit(0)

#    def postOptions(self):
#        """I am called after the options are parsed.
#
#        Override this method in your subclass to do something after
#        the options have been parsed and assigned, like validate that
#        all options are sane.
#        """
#        self.readConfigParserFile()
#        self.setUpPlugins()

#    def readConfigParserFile(self):
#        self.cp = ConfigParser()
#        self.cp.read(self['config'])

#    def setUpPlugins(self):
#        try:
#            items = self.cp.items('plugins')
#            kwargs = dict(items)
#        except Exception, ex:
#            warnings.warn("Failed to setup default pyGridWare.plugins: %s" %str(ex))
#        else:
#            plugin.SetUpDefaults(**kwargs)


class SimpleServiceMaker(object):
    implements(IServiceMaker, IPlugin)
    tapname = "wsrf"
    description = "No security"
    options = SimpleOptions

    def makeService(self, options):
        """
        Construct a TCPServer
        """
        from pyGridWare.utility.log import gridLog

        gridLog(event='pyGridWare.plugins.SimpleServiceMaker.makeService')

        root = Resource()
        Site.protocol = ChunkedHTTPChannel
        siteFactory = Site(root)

        service = internet.TCPServer(int(options["port"]), siteFactory, 
                      interface=options["interface"])

        root.__class__.server = service
        
        # Initialize Root Function
        tree = options['tree']
        log.msg('call initialization function: %s' %tree, debug=True)
        
        args = tree.split('.')
        mod = __import__('.'.join(args[:-1]))
        for attr in args[1:]:
            mod = getattr(mod, attr)
            
        func = mod    
        assert callable(func), 'initialization function must be callable'
        func(root)

        return service


# Now construct an object which *provides* the relevant interfaces
# The name of this variable is irrelevant, as long as there is *some*
# name bound to a provider of IPlugin and IServiceMaker.

serviceMaker = SimpleServiceMaker()



