############################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
import os, random, types, socket
from twisted.python import log as _log 

from ZSI.wstools.logging import gridLog as _GL
from ZSI.version import Version as _ZV
from pyGridWare.version import Version as _PV

from twisted.python.log import theLogPublisher
from twisted.python import failure
import datetime
from cStringIO import StringIO
# note: numerically, DEBUG < INFO < WARN < ERROR < CRITICAL
#from logging import DEBUG, INFO, WARN, ERROR, CRITICAL
from ZSI.wstools.logging import gridLog

"""
Utilities for instrumenting code.

DEFAULT_GRIDLOG_ON -- by default gridLog on/off
DEFAULT_GRIDLOG_DEST -- default URL for gridLog

"""
DEFAULT_GRIDLOG_ON = '1'
DEFAULT_GRIDLOG_DEST = 'gridlog-udp://netlogger.lbl.gov:15100'

os.environ['GRIDLOG_ON'] = os.environ.get('GRIDLOG_ON', DEFAULT_GRIDLOG_ON)
os.environ['GRIDLOG_DEST'] = os.environ.get('GRIDLOG_DEST', DEFAULT_GRIDLOG_DEST)


def gridLog(**kw):
    _GL(zsi='v%d.%d.%d' %_ZV, pyGridWare='v%d.%d.%d' %_PV, **kw)


def instrument(func):
    """@instrument: decorator for instrumenting bound and
    class methods.  static methods should not be used.
    """
    try:
        if not bool( os.environ.get('GRIDLOG_ON', False) ):
            return func
    except:
        return func

    def gridlog_wrapper_func(self, *args, **kw):
        if type(self) in (type, types.ClassType):
            event = 'auto.%s.%s' %(self.__name__, func.__name__)
        else:
            event = 'auto.%s.%s' %(self.__class__.__name__, func.__name__)

        callid = random.random()
        _log.gridlog(event='start.%s' %event, callid=callid, prog="pyGridWare")
        v = func(self, *args, **kw)
        _log.gridlog(event='end.%s' %event, callid=callid, prog="pyGridWare")
        return v

    return gridlog_wrapper_func


def _gridlog(self, **kw):
    """ Gridlog a new message.

    keywords:

        event -- 
        date -- timestamp
        level -- logging level (see levels below)
        status -- integer status code
        gid -- global grid identifier 
        gid, cgid -- parent/child identifiers
        prog -- program name

    """
    if self.globserver is None: 
        return

    try:
        self.globserver(**kw)
    except KeyboardInterrupt:
        # Don't swallow keyboard interrupt!
        raise
    except UnicodeEncodeError:
        raise
    except:
        self.globserver = None
        _log.err(failure.Failure(), "Log gridlog observer failed, removing." )


# Add GridLog message
theLogPublisher.gridlog = _gridlog
_log.gridlog = theLogPublisher.gridlog
theLogPublisher.globserver = None


#def startLoggingWithObserver(observer, setStdout=1):
#    """
#    Initialize logging to a specified observer. If setStdout is true
#    (defaults to yes), also redirect sys.stdout and sys.stderr
#    to the specified file.
#    """
#    _log.startLoggingWithObserver(observer, setStdout=1)
#    if not isinstance(observer, GridLogObserver):
#        return
#
#    _log.removeObserver(observer)
#    
#    assert callable(observer)
#    theLogPublisher.globservers.append(observer)
#

def startGridLogging():
    theLogPublisher.globserver = gridLog

