###########################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################

# $Id: resource.py 1289 2007-01-19 01:34:46Z boverhof $

import os,sys,warnings
from cStringIO import StringIO

# twisted 
from zope.interface import classProvides, implements, Interface
from twisted.web.error import NoResource
from twisted.web.server import NOT_DONE_YET
from twisted.web.resource import Resource, IResource
from twisted.python import log, failure
from twisted.python.runtime import platformType, platform

# ZSI
from ZSI import _get_element_nsuri_name, EvaluateException, ParseException
from ZSI.parse import ParsedSoap
from ZSI.writer import SoapWriter
from ZSI import fault
from ZSI.address import Address
from ZSI.ServiceContainer import WSActionException, WSActionNotSpecified
from ZSI.twisted.WSresource import WSResource as _WSResource,\
    CallbackChainInterface, DataHandler, DefaultHandlerChain, WSAddressHandler,\
    HandlerChainInterface

from pyGridWare.utility.http import GetURLFromRequest
try:
    from pyGridWare.utility.dom import DomletteReader, DomletteElementProxy
except ImportError: 
    DomletteReader = DomletteElementProxy = None
    
from pyGridWare.wsrf.faults import BaseFault
from pyGridWare.resource.interfaces import IWSResource

# 
# Swap in 4Suite Domlette for Performance and XPath support
# 
DataHandler.readerClass = DomletteReader
DataHandler.writerClass = DomletteElementProxy

from twisted.internet import defer, error
from twisted.cred import credentials, checkers



class CallbackHandler:
    classProvides(CallbackChainInterface)

    @classmethod
    def processRequest(cls, ps, **kw):
        """invokes callback that should return a (request,response) tuple.
        representing the SOAP request and response respectively.
        ps -- ParsedSoap instance representing HTTP Body.
        request -- twisted.web.server.Request
        """
        resource = kw['resource']
        request = kw['request']
        method = resource.root[_get_element_nsuri_name(ps.body_root)]
        if method is None:
            raise UnknownRequestException,\
                'failed to map request to a method: action(%s), root%s'\
                %(action,_get_element_nsuri_name(ps.body_root))
              
        req_pyobj,rsp_pyobj =\
            getattr(resource, method)(ps, ps.address, request=request)
        
        return rsp_pyobj
      

class HandlerChainFactory:
    protocol = DefaultHandlerChain
    
    @classmethod
    def newInstance(cls):
        return cls.protocol(CallbackHandler, DataHandler, WSAddressHandler())

    
    
class WSResource(_WSResource):
    '''
    class variables:
        encoding  
        soapAction -- Must override to set input Action values.
           Dictionary of soapAction keys, and operation name values.
           These are specified in the WSDL soap bindings. There must be a 
           class method matching the operation name value.  If WS-Action is
           used the keys are WS-Action request values, according to the spec
           if soapAction and WS-Action is specified they must be equal.
           
        wsAction -- Must override to set output Action values.
           Dictionary of operation name keys and WS-Action 
           response values.  These values are specified by the portType.

        root -- dictionary of root element keys, and operation name values.
        
        wsdl -- WSDL file path
    '''
    implements(IWSResource)
    
    soapAction = {}
    wsAction = {}
    root = {}
    wsdl = None
    factory = HandlerChainFactory

    def render_GET(self, request):
        """try to return WSDL and manipulate the soap:address location
        and wsdl:import, xsd:import, xsd:include
        """
        s = StringIO()
        request.path
        wsdl = self.wsdl

        if wsdl is None or not os.path.isfile(wsdl):
            raise NoResource('request(%s), no file(%s)' %
                (request.path, wsdl))
            
        import re
        from urlparse import urlunparse, urlparse
        
        scheme = 'http'
        if request.transport.TLS != 0: 
            scheme = 'https'
        
        netloc = '%s:%d' %(request.host.host, request.host.port)
        repl = urlunparse((scheme, netloc, request.path, '', '', ''))
        pat_addr = re.compile(r'<([^ \t\n\r\f\v:]+:)?address\s+location\s*=\s*"https?://[\w+.?]+\S*')
        pat_wsdl_import = re.compile(
             r'<([^ \t\n\r\f\v:]+:)?import\s+(namespace\s*=\s*"\S+"\s+)?location\s*="(../\S+)"')
        pat_xsd_include = re.compile(
             r'<([^ \t\n\r\f\v:]+:)?include\s+location\s*=\s*"(\S+)"')
        pat_xsd_import = re.compile(
            r'<([^ \t\n\r\f\v:]+:)?import\s+(namespace\s*=\s*"\S+"\s+)?schemaLocation\s*="(../\S+)"')
        
        for l in open(wsdl,'r'):
            s.write(l)
            
        def f1(m):
            return repl
        
        def f2(m):
            last = len(m.groups())
            start,end = m.span(last)
            tp = urlparse(m.string[start:end])
            url = repl + '/' + tp[2]
            r = m.string[:start] + url + m.string[end:]
            return r
        
        count = 1
        wsdl = s.getvalue()
        while count:
            #wsdl,count = pat_addr.subn(f1, wsdl, 1)
            wsdl,count = pat_wsdl_import.subn(f2, wsdl)
#        wsdl,count = pat_xsd_import.subn(f2, wsdl)
#        wsdl,count = pat_xsd_include.subn(f2, wsdl)
        
        return wsdl
        
        
        

