#!/usr/bin/env python
############################################################################
# Joshua R. Boverhof, LBNL
# See LBNLCopyright for copyright notice!
###########################################################################
import sys
from twisted.python import log
from twisted.internet import reactor

from ZSI import SoapWriter
from pyGridWare.utility.scripts.client import GetBasicOptParser, GetPortKWArgs, SetUp
from pyGridWare.common import delegationlib as dlib


DESC=\
"""Test script for globus container DelegationFactoryService test, delegates credential 
to remote service and saves EPR in file.  Used with GSI security, either transport or message 
security.\n

Example transport security:
./delegation_client.py --full -uhttps://131.243.2.159:8443/wsrf/services/DelegationFactoryService myepr.xml

Example message security:
./delegation_client.py --full -uhttp://131.243.2.159:8080/wsrf/services/DelegationFactoryService 
      -vhttp://131.243.2.159:8080/wsrf/services/DelegationFactoryService-authService myepr.xml

"""

def main(out, **kw):
    epr = dlib.delegate_cred(**kw)
    sw = SoapWriter(envelope=False)
    sw.serialize(epr)
    print >>out, str(sw)
    reactor.stop()

if __name__ == '__main__':
    op = GetBasicOptParser(usage="usage: %prog [options] file", description=DESC)
    op.add_option("-f", "--full", action="store_true", dest="full",
        default=False, help="flag to set full delegation")
    op.add_option("-g", "--lifetime", action="store", dest="lifetime", type=int,
        default=12, help="lifetime of credential in hours")

    (options, args) = op.parse_args()
    SetUp(options)
    kw = GetPortKWArgs(options)
    out = sys.stdout
    if len(args) == 1:
        out = open(args[0], 'w')

    reactor.callWhenRunning(main, out, isFull=options.full, **kw)
    reactor.run()


