##########################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
import ez_setup
ez_setup.use_setuptools()

import os, sys, warnings, pydoc, inspect
from distutils import log
from os.path import isdir, isfile, join as _join
from os.path import sep as SEP
from ConfigParser import ConfigParser, NoOptionError
from setuptools import setup, find_packages
from setuptools.dist import Distribution

# globals
CONFIG_FILE = 'config.txt'
WSDL_SEC = 'WSDL'
TYPES_SEC = 'TYPES'
PERSISTENT_SEC = 'Persistent'
GENERATED = 'pyGridWare.generated'

sys.path.append(_join(os.getcwd(), "pyGridWare", "utility", "generate"))
if sys.version_info < (2, 4):
    raise SystemExit("Python 2.4 or later is required")


def _version():
    cf = ConfigParser()
    cf.read('setup.cfg')
    major = cf.getint('version', 'major')
    minor = cf.getint('version', 'minor')
    patchlevel = cf.getint('version', 'patchlevel')
    candidate = cf.getint('version', 'candidate')

    version = "%d.%d.%d" % (major, minor, patchlevel)
    if candidate:
        version += '_rc%d' % candidate

    text = ('# Auto-generated file; do not edit\n'
            'Version = (%d, %d, %d)\n' % (major, minor, patchlevel))

    vmod = 'pyGridWare/version.py'
    if not os.path.exists(vmod) or open(vmod, 'r').read() != text:
        open(vmod, 'w').write(text)
        
    return version


def _pkg2file(name, asdir=False):
    """utility function, 
    name --  package/module name
    """
    if asdir: return _join(*name.split('.'))
    return '%s.py' %_join(*name.split('.'))


def _done():
    """utility function, has code been generated.
    """
    return isfile(_pkg2file('%s.__init__' %GENERATED))


def _create_generated_files(items, cp=None, **kw):
    import wsdl2web
    kwargs = dict(force=True, mod='pyGridWare.generated')
    kwargs.update(kw)

    for opt,url in items:
        print url

        simple = True
        #if cp is not None:
        #    try: 
        #        simple = not cp.getboolean(PERSISTENT_SEC, opt)
        #    except NoOptionError: 
        #        pass

        wsdl2web.install(url, simple=simple, smod=opt.strip(), **kwargs)

   
def _make_generated_package():
    """Returns False if "generated" package is already present and doesnt need
    to be regenerated, else generates it and returns True. 
    """
    if not len(sys.argv) or ( _done() and sys.argv[1] not in ('install', 'build', 'develop') ):
        return False


    cp = ConfigParser()
    cp.read(CONFIG_FILE)

    # attempt to remove generated
    d = _pkg2file(GENERATED)
    if isdir(d):
        try:
            _remove_generated_files(d)
        except OSError:
            print>>sys.stderr, "ERROR: Must Manually Remove Generation Directory (rm -rf %s)" %d
            sys.exit(-1)
 
    # Create types
    print "|" + "--"*22 + "|"
    print "|  %-40s  |" %"<< Core XSD Schema code generation >>"
    print "|" + "--"*22 + "|"

    _create_generated_files(cp.items(TYPES_SEC), services=False, stubs=False, xsd=True, cp=cp, web2=False)
    print
        
    # Create services/stubs/types
    print "|" + "--"*22 + "|"
    print "|  %-40s  |" %"<< Core WSDL code generation >>"
    print "|" + "--"*22 + "|"

    _create_generated_files(cp.items(WSDL_SEC), services=True, stubs=True, cp=cp, web2=False)
    print

    return True 


def _remove_generated_files(d):
    """
    Utility method to clean up the geneated files.
    """
    for root, dirs, files in os.walk(d, topdown=False):
        for f in files:
            os.remove(_join(root, f))
        for d in dirs:
            os.rmdir(_join(root, d))

    os.rmdir(root)


def main():
    dependency_links=[
        "http://sourceforge.net/project/showfiles.php?group_id=26590&package_id=30660",
        "http://sourceforge.net/project/showfiles.php?group_id=6473&package_id=6541&release_id=286213",
        "http://dsd.lbl.gov/gtg/projects/download/eggs",
    ]

    # Indirection so that setuptools grabs "setup_requires" dependencies
    # before calling "_make_generated_package()"
    # Make setuptools grab PyXML, before ZSI 
    Distribution(
        dict(setup_requires="PyXML >= 0.8.3",
             dependency_links=dependency_links,
        )
    )
    Distribution(
        dict(setup_requires="ZSI >= 2.1.dev_r1393",
             dependency_links=dependency_links,
        )
    )

    _make_generated_package()

    setup(name="pyGridWare", 
          version=_version(),
          author="Joshua Boverhof", 
          author_email="JRBoverhof@lbl.gov",
          url="http://dsd.lbl.gov/gtg/projects/pyGridWare",
          download_url="http://dsd.lbl.gov/gtg/projects/download/download_info.html",
          description="Python WS-Resource Framework and GSS Security",
          maintainer="Joshua Boverhof", 
          maintainer_email="JRBoverhof@lbl.gov",

          classifiers = [
              "Development Status :: 5 - Production/Stable",
              "Intended Audience :: Developers",
              "Intended Audience :: Science/Research",
              "License :: OSI Approved :: BSD License",
              "Programming Language :: Python",
              "Natural Language :: English",
              "Operating System :: Microsoft :: Windows",
              "Operating System :: POSIX"
          ],
          packages=find_packages(),

          scripts=["tools/wsdl2web",
                   ],

          zip_safe=False,
          distclass=Distribution,

          entry_points={\
              'distutils.commands': ['pydoc = pyGridWare.utility.command.docker:PyDocForGeneratedModules'],
              'console_scripts': ['pyrft = pyGridWare.clients.pyrft:main',
                              'pydelegate_credential = pyGridWare.clients.pydelegate_credential:main',
                              'pywsrf_query = pyGridWare.clients.pywsrf_query:main']
          },
          
          setup_requires=[
              "setuptools >= 0.6c5",
          ],
          install_requires=[
              "ZSI >= 2.1.dev_r1392", "4Suite-XML >= 1.0.2"
          ],

          extras_require={ 
              "MessageSecurity":["M2Crypto>=0.17", "pyGsi>=0.2"],
              "https":["M2Crypto>=0.17",], 
          },

          dependency_links=dependency_links, 
    )


if __name__ == '__main__': 
    main()
