############################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
import unittest
from urlparse import urlparse, urlunparse
from pyGridWare.utility.log import gridLog


class SubscribeMarker:
    """Marking this TestCase as a Subscription, it must
    initialize a notification consumer to gather notifications
    """


class WSRFTestCase(unittest.TestCase):
    """
    class attributes:
        path -- the path portion of the url of the service
    """
    path = None

    def __init__(self, methodName='runTest'):
        """Instance Attributes
        rsp -- response
        epr -- endpoint reference
        client_kw -- client keyword args passed via locatory factory method.
        port -- 
        consumer -- potentially may need to keep track of a consumer service instance
        """
        unittest.TestCase.__init__(self, methodName)
        self.rsp = None
        self.epr = None
        self.client_kw = None
        self.port = None
        self.consumer = None

    def getConsumerService(self):
        if self.consumer is None: return
        self.failUnless(self.consumer.consumer is not None, 'SubscribeTestCase must be setUp')
        return self.consumer.consumer

    def getMethodToCall(self):
        """return method to call, must be callable and 
        takes request as argument.
        """
        raise NotImplementedError, 'this method must be implemented'
        
    def NewRequestInstance(self):
        """return request instance.
        """
        raise NotImplementedError, 'this method must be implemented'

    def getEPR(self):
        """if factory method, return EPR from response.
        """
        return self.epr

    def setEPR(self, epr):
        """set EndPointReference
        """
        self.epr = epr

    def runTest(self):
        kw = {}
        msg = self.NewRequestInstance()
        method = self.getMethodToCall()
        if msg is None:
            raise
        if method is None or not callable(method):
            raise

        e = 'pyGridWare.test.%s.%s' %(self.__class__.__module__, self.__class__.__name__)
        gridLog(event='%s.start' %e, prog='runTests.py')
        self.rsp = method(msg, **kw)
        gridLog(event='%s.end' %e, prog='runTests.py')

    def run(self, result=None, **kw):
        """replace path
        """
        path = self.path
        if path is None:
            raise RuntimeError, "ERROR path is None, eg. /wsrf/services/CounterService"

        self.client_kw = kw
        scheme,netloc,path,params,query,fragment = urlparse(self.client_kw['url'])
        self.client_kw['url'] = urlunparse((scheme,netloc,self.path,'','',''))
        unittest.TestCase.run(self, result)

    __call__ = run


class WSRFTestSuite(unittest.TestSuite):
    def __init__(self, tests=()):
        unittest.TestSuite.__init__(self, tests=tests)

    def run(self, result, **kw):
        """Factory method first.  Each factory method call will change the EPR
        for subsequent calls.
        """
        epr = None
        for test in self._tests:
            if result.shouldStop: 
                break

            if isinstance(test, WSRFTestCase):
                # counter testcase
                test.setEPR(epr)
                test(result, **kw)
                epr = test.getEPR()

                if isinstance(test, SubscribeMarker):
                    # consumer only retained w/i a TestSuite
                    kw['consumer'] = test.getConsumerService()
            else:
                # counter suite
                if kw.has_key('consumer'): 
                    kw['consumer'].stop()
                    del kw['consumer']
                test(result, **kw)
                
        return result
    __call__ = run


import benchmarks, bigfat, counter, authz, performance, digsig, delegation, persistent
__all__=['benchmarks', 'bigfat', 'counter', 'authz', 'performance', 'digsig', 'delegation', 'persistent',]
