#!/usr/bin/env python
##########################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
import warnings
from suites import WSRFTestCase, WSRFTestSuite
try:
    from pyGridWare.common import delegationlib as dlib
except:
    warnings.warn('delegation test suite not available because of import errors')

# DelegationFactoryService and DelegationService are core WSDL
WSDL = None

class DelegationFactoryTestCase(WSRFTestCase):
    path = '/wsrf/services/GSIPortalDelegationFactoryService'
    
    def tearDown(self):
        self.shouldStop = False
        
    def runTest(self):
        self.epr = dlib.delegate_cred(isFull=True, lifetime=12, **self.client_kw)
        self.shouldStop = True
        

class GetRPTestCase(WSRFTestCase):
    path = '/wsrf/services/GSIPortalDelegationFactoryService'

    def tearDown(self):
        self.shouldStop = False

    def setUp(self):
        import pyGridWare.generated.stubs.delegation_factory_service as client
        loc = client.DelegationFactoryServiceLocator()
        kw = self.client_kw
        self.port = loc.getDelegationFactoryPortTypePort(endPointReference=self.epr, **kw)
        self.shouldStop = True

    def getMethodToCall(self):
        return self.port.GetResourceProperty

    def NewRequestInstance(self):
        import pyGridWare.generated.stubs.delegation_factory_service as client
        qname = ('http://www.globus.org/08/2004/delegationService', 'CertificateChain')
        return client.GetResourcePropertyRequest(qname)

    def runTest(self):
        WSRFTestCase.runTest(self)
        RefreshTestCase.bstokens = self.rsp.Any


class DelegationTestCase(WSRFTestCase):
    path = '/wsrf/services/GSIPortalDelegationService'
    
    def tearDown(self):
        self.shouldStop = False

    def setUp(self):
        import pyGridWare.generated.stubs.delegation_service as client
        loc = client.DelegationServiceLocator()
        kw = self.client_kw
        self.port = loc.getDelegationPortTypePort(endPointReference=self.epr, **kw)
        self.shouldStop = True


class RefreshTestCase(DelegationTestCase):
    bstokens = None

    def getMethodToCall(self):
        return self.port.refresh

    def NewRequestInstance(self):
        import pyGridWare.generated.stubs.delegation_service as client
        msg = client.RefreshRequest()
        msg.Any = RefreshTestCase.bstokens
        return msg

        

def default():
    """Create new SSL Context in order to use a different credential,
    then try to access a service resource instance created with the
    original credential and protected by ResourcePortal.

    Receives a "ResourceUnknownFault", because we can not see the
    the resource as a different user.
    """
    return WSRFTestSuite(tests=(DelegationFactoryTestCase(), GetRPTestCase(), RefreshTestCase()))
        
       
