#!/usr/bin/env python
##########################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
import unittest, os
from ZSI import ZSIException
import counter
from os.path import abspath, join, sep
_resolvepath = lambda f: abspath(join(*f.split('/')))
WSDL = counter.WSDL
PATH = '/wsrf/services/SecureCounterService'


class SecureConversationCounterTestSuite(unittest.TestSuite):
    """Sets up a security context using Secure Conversation Client, 
    then sets a signature handler for the client to use this context.
    """
    path = '/wsrf/services/SecureConversationService'
    def __init__(self, tests=()):
        unittest.TestSuite.__init__(self, tests=tests)

    def run(self, result, contextFactory=None, **kw):
        """Factory method first.  Each factory method call will change the EPR
        for subsequent calls.
        """
        #     
        # TODO: Little Ugly have to setup ManagerHome
        #     before creating SecureConversationClient
        #     
        from pyGridWare.security.m2.ResourceHome import ManagerHome
        from pyGridWare.plugin import GetPlugin
        from pyGridWare.resource.interfaces import IResourceHomeFactory
        home = ManagerHome.getInstance(GetPlugin(IResourceHomeFactory))

        #from pyGridWare.utility.gsi import GSIClientContextFactory
        from pyGridWare.security.m2.M2SignatureHandler import M2SignatureHandler
        from pyGridWare.security.m2.Authenticate import SecureConversationClient

        #factory = GSIClientContextFactory()
        sc = SecureConversationClient()
        url = '%s%s' %(kw['url'], self.path)
        ctx_id = sc.exchange(contextFactory.getContext(), url=url,
                    wsAddressURI=kw['wsAddressURI'])

        kw['sig_handler'] = handler = M2SignatureHandler()
        handler.setSecureConversation(ctx_id)

        epr = None
        for test in self._tests:
            if result.shouldStop: 
                break

            if isinstance(test, counter.CounterTestCase):
                # counter testcase
                test.setEPR(epr)
                test(result, contextFactory=contextFactory, **kw)
                epr = test.getEPR()

                if isinstance(test, counter.SubscribeMarker):
                    # consumer only retained w/i a TestSuite
                    kw['consumer'] = test.getConsumerService()
            else:
                # counter suite
                if kw.has_key('consumer'): 
                    kw['consumer'].stop()
                    del kw['consumer']
                test(result, contextFactory=contextFactory, **kw)
                
        return result
    __call__ = run


def SecureCounterTestSuite():
    """testing secure conversation and secure counter
    """
    tests = (counter.CreateTestCase(), counter.AddTestCase(), counter.DestroyTestCase())

    for t in tests: 
        t.path = PATH

    return SecureConversationCounterTestSuite(tests)

default = SecureCounterTestSuite


